from datetime import datetime
from typing import Any, Iterator

from ...models import BaseSnippet, ResourceType, Search
from ...schemas import (
    PageInfo,
    SearchFilter,
    SearchOptionalParameters,
    SearchPart,
    YouTubeRequest,
    YouTubeResponse,
)
from ..resource import YouTubeResource


class YouTubeSearch(YouTubeResource):
    def __init__(self, youtube_client: Any) -> None:
        super().__init__(youtube_client)
        self.search_schema: YouTubeRequest = None
        self.next_page_token: str = None

    def parse_id(self, id_data: dict[str, str]) -> dict[str, str]:
        resource_id: str = ''
        resource_type: ResourceType = None
        if id_data['kind'] == 'youtube#playlist':
            resource_id = id_data['playlistId']
            resource_type = 'playlist'
        elif id_data['kind'] == 'youtube#video':
            resource_id = id_data['videoId']
            resource_type = 'video'
        else:
            resource_id = id_data['channelId']
            resource_type = 'channel'
        parsed_id_data: dict[str, str] = {
            'resource_id': resource_id,
            'resource_type': resource_type,
        }
        return parsed_id_data

    def parse_snippet(
        self, snippet_data: dict[str, str | datetime]
    ) -> dict[str, str | datetime]:
        base_snippet: BaseSnippet = self.parse_base_snippet(snippet_data)
        parsed_snippet: dict[str, Any] = base_snippet.model_dump()
        parsed_snippet['channel_id'] = snippet_data['channelId']
        parsed_snippet['channel_title'] = snippet_data['channelTitle']
        parsed_snippet['live_broadcast_content'] = snippet_data['liveBroadcastContent']
        parsed_snippet['publish_time'] = snippet_data['publishTime']
        return parsed_snippet

    def parse_item(self, item: dict) -> Search:
        id_data: dict = self.parse_id(item['id'])
        snippet_data: dict = self.parse_snippet(item['snippet'])
        parsed_item: Search = Search(
            resource_id=id_data['resource_id'],
            resource_type=id_data['resource_type'],
            description=snippet_data['description'],
            thumbnails=snippet_data['thumbnails'],
            title=snippet_data['title'],
            channel_id=snippet_data['channel_id'],
            published_at=snippet_data['published_at'],
            channel_title=snippet_data['channel_title'],
            live_broadcast_content=snippet_data['live_broadcast_content'],
            publish_time=snippet_data['publish_time'],
        )
        return parsed_item

    def search(self, search_schema: YouTubeRequest) -> YouTubeResponse:
        search_dict: dict[str, Any] = self.create_request_dict(search_schema)
        search_request = self.youtube_client.search().list(**search_dict)
        search_response = search_request.execute()
        return self.parse_youtube_response(search_response)

    def find_channel_by_name(self, display_name: str) -> str:
        part: SearchPart = SearchPart()
        optional_parameters: SearchOptionalParameters = SearchOptionalParameters(
            q=display_name, type=['channel']
        )
        search_request: YouTubeRequest = YouTubeRequest(
            part=part, optional_parameters=optional_parameters
        )
        return self.search(search_request)

    def __iter__(self):
        return self

    def __next__(self) -> list[Search]:
        self.search_schema.optional_parameters.pageToken = self.next_page_token
        search_results: YouTubeResponse = self.search(self.search_schema)
        self.next_page_token = search_results.nextPageToken
        return search_results.items

    def get_search_iterator(self, search_schema: YouTubeResponse) -> Iterator:
        self.search_schema = search_schema
        return self



from .activity import Activity, ActivityContentDetails, ActivitySnippet
from .activity_type import (
    Activities,
    ActivityType,
    Bulletin,
    ChannelItem,
    CommentAdded,
    PromotedItem,
    Reccomendation,
    ResourceId,
    ShareToSocial,
    UserSubscribedToChannel,
    VideoAddedToFavorite,
    VideoAddedToPlaylist,
    VideoLiked,
    VideoUploaded,
)
from .caption import Caption
from .channel import (
    Channel,
    ChannelContentDetails,
    ChannelSnippet,
    ChannelStatistics,
    ChannelStatus,
)
from .channel_banner import ChannelBanner
from .channel_section import ChannelSection, ChannelSectionType
from .comment import Author, Comment
from .comment_thread import CommentThread
from .content_details import BaseContentDetails
from .language import Language, LanguageSnippet
from .localized import Localized
from .playlist import Playlist
from .playlist_item import PlaylistItem
from .region import Region, RegionSnippet
from .resource import Resource
from .resource_type import ResourceType
from .scopes import Scopes
from .search import Search
from .snippet import BaseSnippet
from .status import BaseStatus
from .subscription import Subscription
from .thumbnail import Thumbnail, ThumbnailResolution
from .video import ContentDetails, Statistics, Status, Video
from .video_abuse_report_reason import VideoAbuseReportReason, VideoAbuseReportReasonSnippet
from .video_category import VideoCategory, VideoCategorySnippet
from .watermark import WaterMark



from enum import Enum, auto

from pydantic import BaseModel

from .resource_type import ResourceType


class Activities(Enum):
    channelItem = 'channelItem'
    comment = 'comment'
    favorite = 'favorite'
    like = 'like'
    playlistItem = 'playlistItem'
    promotedItem = 'promotedItem'
    recommendation = 'recommendation'
    social = 'social'
    subscription = 'subscription'
    upload = 'upload'
    bulletin = 'bulletin'


class ResourceId(BaseModel):
    kind: str
    id: str


class ActivityType(BaseModel):
    pass


class VideoUploaded(ActivityType):
    video_id: str


class VideoLiked(ActivityType):
    resource_id: ResourceId


class VideoAddedToFavorite(ActivityType):
    resource_id: ResourceId


class CommentAdded(ActivityType):
    resource_id: ResourceId


class UserSubscribedToChannel(ActivityType):
    resource_id: ResourceId


class VideoAddedToPlaylist(ActivityType):
    video_id: str
    playlist_id: str


class Reccomendation(ActivityType):
    resource_id: ResourceId
    reason: str
    seed_resource_id: ResourceId


class ShareToSocial(ActivityType):
    type: str
    resource_id: ResourceId
    author: str
    reference_url: str
    image_url: str


class ChannelItem(ActivityType):
    resource_id: ResourceId


class PromotedItem(ActivityType):
    pass


class Bulletin(ActivityType):
    pass



from typing import Optional

from pydantic import BaseModel, Field

from .youtube_request import Filter, OptionalParameters, Part


class CommentThreadPart(Part, BaseModel):
    part: list[str] = Field(default=['id', 'replies', 'snippet'])


class CommentThreadFilter(Filter, BaseModel):
    allThreadsRelatedToChannelId: Optional[str] = ''
    id: Optional[list[str]] = Field(default_factory=list)
    channelId: Optional[str] = None
    videoId: Optional[str] = None


class CommentThreadOptionalParameters(OptionalParameters):
    maxResults: Optional[int] = None
    moderationStatus: Optional[int] = None
    order: Optional[str] = None
    pageToken: Optional[str] = None
    searchTerms: Optional[list[str]] = Field(default_factory=list)
    textFormat: Optional[str] = None



from datetime import datetime

from pydantic import BaseModel

from .localized import Localized
from .thumbnail import Thumbnail


class BaseSnippet(BaseModel):
    title: str
    description: str
    thumbnails: Thumbnail
    published_at: datetime



from typing import Optional

from pydantic import BaseModel, Field


class Part(BaseModel):
    pass


class Filter(BaseModel):
    id: Optional[list[str]] = Field(default_factory=list)


class OptionalParameters(BaseModel):
    pass


class YouTubeRequest(BaseModel):
    part: Part
    filter: Optional[Filter] = Filter()
    optional_parameters: Optional[OptionalParameters] = OptionalParameters()



from typing import Optional

from pydantic import BaseModel


class ThumbnailResolution(BaseModel):
    url: str
    width: int
    height: int


class Thumbnail(BaseModel):
    default: Optional[ThumbnailResolution] = None
    medium: Optional[ThumbnailResolution] = None
    high: Optional[ThumbnailResolution] = None
    standard: Optional[ThumbnailResolution] = None
    maxres: Optional[ThumbnailResolution] = None



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field

from .resource import Resource
from .thumbnail import Thumbnail


class RegionRestriction(BaseModel):
    allowed: list[str] = Field(default_factory=list)
    blocked: list[str] = Field(default_factory=list)


class ContentRating(BaseModel):
    pass


class Status(BaseModel):
    upload_status: str
    failure_reason: str
    rejection_reason: str
    privacy_status: str
    publish_at: datetime
    license: str
    embedabble: bool
    public_stats_viewable: bool
    made_for_kids: bool
    self_declared_made_for_kids: bool


class Statistics(BaseModel):
    views_count: int
    likes_count: int
    comments_count: int


class Localized(BaseModel):
    title: str
    description: str


class Snippet(BaseModel):
    title: str
    channel_id: str
    channel_title: str
    description: str
    thumbnails: Thumbnail
    tags: list[str] = Field(default_factory=list)
    published_at: datetime
    category_id: str
    live_broadcast_content: str
    default_language: str = None
    localized: Localized


class ContentRatings(BaseModel):
    pass


class ContentDetails(BaseModel):
    duration: str
    dimension: str
    definition: str
    caption: str
    licensed_content: bool
    # content_ratings: ContentRatings
    projection: str
    has_custom_thumbnail: Optional[bool] = None
    region_restriction: Optional[RegionRestriction] = RegionRestriction()


class Video(Resource):
    id: str
    snippet: Snippet
    content_details: ContentDetails
    # status: Status
    statistics: Statistics



from enum import Enum, auto

from pydantic import BaseModel, Field

from .resource import Resource


class ChannelSectionSnippet(BaseModel):
    type: str
    channel_id: str
    title: str
    position: int


class ChannelSectionContentDetails(BaseModel):
    playlists: list[str] = Field(default_factory=list)
    channels: list[str] = Field(default_factory=list)


class ChannelSection(Resource):
    snippet: ChannelSectionSnippet
    content_details: ChannelSectionContentDetails


class ChannelSectionType(Enum):
    allPlaylists = 'allPlaylists'
    completedEvents = 'completedEvents'
    liveEvents = 'liveEvents'
    multipleChannels = 'multipleChannels'
    multiplePlaylists = 'multiplePlaylists'
    popularUploads = 'popularUploads'
    recentUploads = 'recentUploads'
    singlePlaylist = 'singlePlaylist'
    subscriptions = 'subscriptions'
    upcomingEvents = 'upcomingEvents'



from datetime import datetime
from typing import Optional

from pydantic import BaseModel

from ..models.resource import Resource
from .resource_id import ResourceId
from .status import Status
from .thumbnail import Thumbnail


class Snippet(BaseModel):
    title: str
    channel_id: str
    channel_title: str
    description: str
    thumbnails: Thumbnail
    playlist_id: str
    resource_id: ResourceId
    video_owner_channel_title: Optional[str] = None
    video_owner_channel_id: Optional[str] = None
    published_at: Optional[datetime] = None
    position: Optional[int] = 0


class ContentDetails(BaseModel):
    video_id: str
    video_published_at: Optional[datetime] = None
    note: Optional[str] = ''


class PlaylistItem(Resource):
    id: str
    snippet: Snippet
    content_details: Optional[ContentDetails] = None
    status: Optional[Status] = None



from typing import Optional

from pydantic import BaseModel, Field

from .youtube_request import Filter, OptionalParameters, Part


class PlaylistItemPart(Part):
    part: list[str] = Field(default=['contentDetails', 'id', 'snippet', 'status'])


class PlaylistItemFilter(Filter):
    playlistId: Optional[str] = ''


class PlaylistItemOptionalParameters(OptionalParameters):
    maxResults: Optional[int] = None
    onBehalfOfContentOwner: Optional[str] = ''
    videoId: Optional[str] = ''
    pageToken: Optional[str] = ''


class VideoResourceId(BaseModel):
    videoId: str
    kind: Optional[str] = 'youtube#video'


class CreatePlaylistItemSnippet(BaseModel):
    playlistId: str
    resourceId: VideoResourceId
    position: Optional[int] = 0


class CreatePlaylistItem(BaseModel):
    snippet: CreatePlaylistItemSnippet



from typing import Optional

from pydantic import BaseModel, Field

from .youtube_request import Filter, OptionalParameters, Part


class ChannelSectionPart(Part):
    part: list[str] = Field(default=['contentDetails', 'id', 'snippet'])


class ChannelSectionFilter(Filter):
    channelId: Optional[str] = None
    id: Optional[str] = None
    mine: Optional[bool] = None


class ChannelSectionOptionalParameters(OptionalParameters):
    onBehalfOfContentOwner: Optional[str] = None


class InsertChannelSectionSnippet(BaseModel):
    type: str
    title: Optional[str] = None
    position: Optional[int] = None


class InsertChannelSectionContentDetails(BaseModel):
    playlists: list[str] = Field(default_factory=list)
    channels: list[str] = Field(default_factory=list)


class InsertChannelSection(BaseModel):
    snippet: InsertChannelSectionSnippet
    content_details: Optional[InsertChannelSectionContentDetails] = None



from typing import Optional

from pydantic import Field

from .youtube_request import Filter, OptionalParameters, Part


class ChannelPart(Part):
    part: list[str] = Field(
        default=[
            'contentDetails',
            'id',
            'localizations',
            'snippet',
            'statistics',
            'status',
        ]
    )


class ChannelFilter(Filter):
    forUsername: Optional[str] = None
    id: Optional[list[str]] = Field(default_factory=list)
    managedByMe: Optional[str] = None
    mine: Optional[bool] = None


class ChannelOptionalParameters(OptionalParameters):
    h1: Optional[str] = None
    onBehalfOfContentOwner: Optional[str] = None
    pageToken: Optional[str] = None
    maxResults: Optional[int] = None



from typing import Optional

from pydantic import BaseModel, Field

from .common import PageInfo
from .resource_schema import Resource


class Part(BaseModel):
    pass


class Filter(BaseModel):
    pass


class OptionalParameters(BaseModel):
    pass


class RequestSchema(BaseModel):
    part: Part
    filter: Optional[Filter] = Filter()
    optional_parameters: Optional[OptionalParameters] = OptionalParameters()


class YouTubeResponse(BaseModel):
    kind: str
    etag: str
    pageInfo: PageInfo
    items: list[Resource] = Field(default_factory=list)
    nextPageToken: Optional[str] = ''
    prevPageToken: Optional[str] = ''



from setuptools import find_packages, setup
from pip._vendor import tomli

# For consistent encoding
from codecs import open
from os import path

# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    LONG_DESCRIPTION = f.read()

with open('pyproject.toml', 'r') as f:
    VERSION = tomli.load(f)['tool']['commitizen']['version']

DESCRIPTION = 'A python library that wraps around the YouTube V3 API. You can use it find and manage YouTube resources including Videos, Playlists, Channels and Comments.'

key_words = [
    'youtube', 'youtube-api', 'youtube comments', 'youtube videos',
    'youtube channels', 'youtube comment thread', 'create youtube playlist'
]

install_requires = [
    'oryks-google-oauth',
    'pydantic',
    'pydantic-settings'
]

setup(
    name='oryks-youtube',
    packages=find_packages(
        include=[
            'youtube',
            'youtube.models',
            'youtube.schemas',
            'youtube.resources',
            'youtube.resources.search',
            'youtube.resources.video',
            'youtube.resources.playlist',
            'youtube.resources.playlist_item',
            'youtube.resources.comment_thread',
            'youtube.resources.channel',
        ]
    ),
    version=VERSION,
    description=DESCRIPTION,
    long_description_content_type='text/markdown',
    long_description=LONG_DESCRIPTION,
    url='https://youtube-wrapper.readthedocs.io/en/latest/index.html',
    author='Lyle Okoth',
    author_email='lyceokoth@gmail.com',
    license='MIT',
    install_requires=install_requires,
    keywords=key_words,
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Operating System :: OS Independent'
    ],
)



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field

from .content_details import BaseContentDetails
from .resource import Resource
from .snippet import BaseSnippet
from .status import BaseStatus


class ChannelSnippet(BaseSnippet):
    custom_url: str
    country: str


class RelatedPlaylists(BaseModel):
    likes: str = Field(
        description="The ID of the playlist that contains the channel's liked videos."
    )
    uploads: str


class ChannelContentDetails(BaseContentDetails):
    related_playlists: RelatedPlaylists


class ChannelStatistics(BaseModel):
    views_count: int
    subscribers_count: int
    hidden_subscribers_count: bool
    videos_count: int


class ChannelStatus(BaseStatus):
    is_linked: bool
    long_uploads_status: str
    made_for_kids: bool


class Channel(Resource):
    content_details: ChannelContentDetails
    statistics: ChannelStatistics
    status: ChannelStatus



from pydantic import BaseModel

from .content_details import BaseContentDetails
from .resource import Resource
from .resource_id import ResourceId
from .snippet import BaseSnippet
from .thumbnail import Thumbnail


class SubscriptionSnippet(BaseSnippet):
    channel_title: str
    channel_id: str
    resource_id: ResourceId


class SubscriptionContentDetails(BaseContentDetails):
    total_item_count: int
    new_item_count: int
    activity_type: str


class SubscriberSnippet(BaseModel):
    title: str
    description: str
    channel_id: str
    thumbnails: Thumbnail


class Subscription(Resource):
    snippet: SubscriptionSnippet
    content_details: SubscriptionContentDetails
    subscriber_snippet: SubscriberSnippet



from enum import Enum


class Scopes(Enum):
    youtube_force_ssl = 'https://www.googleapis.com/auth/youtube.force-ssl'
    youtube_readonly = 'https://www.googleapis.com/auth/youtube.readonly'
    youtube_upload = 'https://www.googleapis.com/auth/youtube.upload'
    youtube = 'https://www.googleapis.com/auth/youtube'
    youtubepartner = 'https://www.googleapis.com/auth/youtubepartner'
